div.quick-uploader {
    position: relative;
}

a#copy_categories {
    display: block;
    padding: 7px;
    border-radius: 5px;
    border: none;
    background-color: #205c90;
    text-decoration: none;
    color: #FFF;
    position: absolute;
    top: 72px;
    cursor: pointer;
}

a#copy_categories:hover {
    color: #FFF!important;
    background-color: #518abc;
}

/* quickupload displayed in an overlay :
   - display files to upload next to each other;
   - clear display.
*/
div.overlay div.qq-uploader li {
    display: inline-block;
    margin-left: 3em;
    margin-right: 1em;
    margin-top: -1em;
    margin-bottom: 2em;
    width: 275px;
    border-radius: 5px;
}
div.overlay div.qq-uploader li input[type="text"] {
    width: 93%;
}
div.overlay div.qq-uploader .select2-container {
    font-size: 125%;
}
div.overlay div.qq-upload-infos {
    font-size: 140%;
    color: #333333;
}